@@ -725,6 +725,17 @@ impl<T: ?Sized, State: RcState> HybridRc<T, State> {
 		unsafe { &(*self.ptr.as_ptr()).meta }
 	}
 
+	/// Provides a reference to the inner `HybridRc` of a `Pin<HybridRc<T>>`
+	///
+	/// # Safety
+	/// The caller must ensure that the reference is not used to move the value out of self.
+	#[inline(always)]
+	unsafe fn pin_get_ref(this: &Pin<Self>) -> &Self {
+		// SAFETY: Pin is repr(transparent) and by contract the caller doesn't use the reference
+		// to move the value.
+		unsafe { &*(this as *const Pin<Self>).cast::<Self>() }
+	}
+
 	/// Returns a mutable reference to the value, without checking for uniqueness.
 	///
 	/// # See also
@@ -774,7 +785,11 @@ impl<T: ?Sized, State: RcState> HybridRc<T, State> {
 	#[must_use]
 	#[inline]
 	pub fn as_ptr(this: &Self) -> *const T {
-		this.data()
+		let ptr = this.ptr.as_ptr();
+
+		// SAFETY: Neccessary for `from_raw()` (when implemented), retains provenance.
+		// Besides that, does basically the same thing as `data()` or `get_mut_unchecked()`.
+		unsafe { ptr::addr_of_mut!((*ptr).data) }
 	}
 
 	/// Creates a new [`Weak`] for the referenced value.
@@ -794,12 +809,37 @@ impl<T: ?Sized, State: RcState> HybridRc<T, State> {
 		Weak { ptr: this.ptr }
 	}
 
+	/// Creates a new [`PinWeak`] for the referenced value.
+	///
+	/// # Example
+	/// ```
+	/// use hybrid_rc::{Rc, Weak};
+	///
+	/// let strong = Rc::pin(42i32);
+	/// let weak = Rc::downgrade_pin(&strong);
+	/// ```
+	#[inline]
+	pub fn downgrade_pin(this: &Pin<Self>) -> PinWeak<T> {
+		// SAFETY: We are not moving anything and we don't expose a non-pinned pointer.
+		let this = unsafe { Self::pin_get_ref(this) };
+		PinWeak(Self::downgrade(this))
+	}
+
 	/// Checks if two `HybridRc`s point to the same allocation.
 	#[inline]
 	pub fn ptr_eq<S: RcState>(this: &Self, other: &HybridRc<T, S>) -> bool {
 		this.ptr.as_ptr() == other.ptr.as_ptr()
 	}
 
+	/// Checks if two pinned `HybridRc`s point to the same allocation.
+	#[inline]
+	pub fn ptr_eq_pin<S: RcState>(this: &Pin<Self>, other: &Pin<HybridRc<T, S>>) -> bool {
+		// SAFETY: we are not moving anything and we don't expose any pointers.
+		let this = unsafe { Self::pin_get_ref(this) };
+		let other = unsafe { HybridRc::<T, S>::pin_get_ref(other) };
+		this.ptr.as_ptr() == other.ptr.as_ptr()
+	}
+
 	/// Gets the approximate number of strong pointers to the inner value.
 	///
 	/// As shared pointers cannot access the local reference counter, `Arc::strong_count()` only
@@ -835,6 +875,15 @@ impl<T: ?Sized, State: RcState> HybridRc<T, State> {
 			}
 	}
 
+	/// Gets the approximate number of strong pointers to the pinned inner value.
+	///
+	#[inline]
+	pub fn strong_count_pin(this: &Pin<Self>) -> usize {
+		// SAFETY: We are not moving anything and we don't expose any pointers.
+		let this = unsafe { Self::pin_get_ref(this) };
+		Self::strong_count(this)
+	}
+
 	/// Gets the number of [`Weak`] pointers to this allocation.
 	///
 	/// Please understand that another thread may change the weak count at any time, including
@@ -860,6 +909,15 @@ impl<T: ?Sized, State: RcState> HybridRc<T, State> {
 		}
 	}
 
+	/// Gets the number of [`PinWeak`] pointers to the pinned inner value.
+	///
+	#[inline]
+	pub fn weak_count_pin(this: &Pin<Self>) -> usize {
+		// SAFETY: We are not moving anything and we don't expose any pointers.
+		let this = unsafe { Self::pin_get_ref(this) };
+		Self::weak_count(this)
+	}
+
 	// Constructs an `RcMeta` structure for a new `HybridRc` allocation
 	#[inline]
 	fn build_new_meta() -> RcMeta {
@@ -959,8 +1017,8 @@ impl<T, State: RcState> HybridRc<T, State> {
 		HybridRc::from_inner(inner.into())
 	}
 
-	/// Creates a new `Pin<Arc<T>>`. If `T` does not implement `Unpin`, then `data` will be pinned
-	/// in memory and unable to be moved.
+	/// Creates a new `Pin<HybridRc<T>>`. If `T` does not implement `Unpin`, then `data` will be
+	/// pinned in memory and unable to be moved.
 	#[inline]
 	pub fn pin(data: T) -> Pin<Self> {
 		unsafe { Pin::new_unchecked(Self::new(data)) }
@@ -1155,6 +1213,26 @@ impl<T: ?Sized> Rc<T> {
 		this.meta().inc_strong_shared();
 		Arc::from_inner(this.ptr)
 	}
+
+	/// Creates a new pinned shared reference for the referenced value.
+	///
+	/// # Example
+	/// ```
+	/// use hybrid_rc::{Rc, Weak};
+	///
+	/// let strong = Rc::pin(42i32);
+	/// let shared = Rc::to_shared_pin(&strong);
+	/// assert!(Rc::ptr_eq_pin(&strong, &shared));
+	/// ```
+	#[inline]
+	pub fn to_shared_pin(this: &Pin<Self>) -> Pin<Arc<T>> {
+		// SAFETY: We are not moving anything, we don't expose a non-pinned pointer,
+		// and we create a Pin-wrapper only for a pinned value.
+		unsafe {
+			let this = Self::pin_get_ref(this);
+			Pin::new_unchecked(Self::to_shared(this))
+		}
+	}
 }
 
 impl<T: ?Sized> Arc<T> {
@@ -1205,6 +1283,32 @@ impl<T: ?Sized> Arc<T> {
 			Some(_) => None,
 		}
 	}
+
+	/// Creates a new pinned local reference for the referenced value.
+	///
+	/// Returns `None` if at least one `Rc` already exists on another thread.
+	///
+	/// **Note:** In `no_std` environments `None` is returned if at least one `Rc` exists on *any*
+	/// thread.
+	///
+	/// # Example
+	/// ```
+	/// use hybrid_rc::{Arc, Weak};
+	///
+	/// let strong = Arc::pin(42i32);
+	/// let local = Arc::to_local_pin(&strong).unwrap();
+	/// assert!(Arc::ptr_eq_pin(&strong, &local));
+	/// ```
+	#[must_use]
+	#[inline]
+	pub fn to_local_pin(this: &Pin<Self>) -> Option<Pin<Rc<T>>> {
+		// SAFETY: We are not moving anything, we don't expose a non-pinned pointer,
+		// and we create a Pin-wrapper only for a pinned value.
+		unsafe {
+			let this = Self::pin_get_ref(this);
+			Some(Pin::new_unchecked(Self::to_local(this)?))
+		}
+	}
 }
 
 impl<T: Clone, State: RcState> HybridRc<T, State> {
@@ -1854,6 +1958,12 @@ impl<T: ?Sized, State: RcState> fmt::Pointer for HybridRc<T, State> {
 	}
 }
 
+/// `HybridRc<T>` is always `Unpin` itself, because the data value is on the heap,
+/// so moving `HybridRc<T>` doesn't move the content even if `T` is not `Unpin`.
+///
+/// This allows unpinning e.g. `Pin<Box<HybridRc<T>>>` but not any `Pin<HybridRc<T>>`!
+impl<T: ?Sized, State: RcState> Unpin for HybridRc<T, State> {}
+
 /// `Weak<T>` represents a non-owning reference to a value managed by a [`HybridRc<T, _>`].
 /// The value is accessed by calling [`upgrade()`] or [`upgrade_local()`] on `Weak`.
 ///
@@ -2075,7 +2185,7 @@ impl<T> Weak<T> {
 	///
 	/// [`ValueDropped`]: UpgradeError::ValueDropped
 	pub fn new() -> Weak<T> {
-		Weak { ptr: senitel() }
+		Self { ptr: senitel() }
 	}
 }
 
@@ -2156,3 +2266,79 @@ impl<T: ?Sized> Drop for Weak<T> {
 // Safety: Like for Arc<T> T: Send + Sync gives all guarantees we need to make Weak Send + Sync.
 unsafe impl<T: ?Sized + Sync + Send> Send for Weak<T> {}
 unsafe impl<T: ?Sized + Sync + Send> Sync for Weak<T> {}
+
+#[repr(transparent)]
+pub struct PinWeak<T: ?Sized>(Weak<T>);
+
+impl<T: ?Sized> PinWeak<T> {
+	/// Attempts to upgrade the pinned weak pointer to a pinned [`Rc`].
+	///
+	/// See [`Weak::upgrade_local()`] for more information.
+	///
+	/// # Errors
+	/// - [`ValueDropped`]: the referenced value has already been dropped.
+	/// - [`WrongThread`]: another thread currently holds `Rc`s for the value.
+	#[inline]
+	pub fn upgrade_local(&self) -> Result<Pin<Rc<T>>, UpgradeError> {
+		Ok(unsafe { Pin::new_unchecked(self.0.upgrade_local()?) })
+	}
+
+	/// Attempts to upgrade the pinned weak pointer to a pinned [`Arc`].
+	///
+	/// See [`Weak::upgrade()`] for more information.
+	///
+	/// # Errors
+	/// - [`ValueDropped`]: the referenced value has already been dropped.
+	#[inline]
+	pub fn upgrade(&self) -> Result<Pin<Arc<T>>, UpgradeError> {
+		Ok(unsafe { Pin::new_unchecked(self.0.upgrade()?) })
+	}
+
+	/// Gets a lower bound to the number of strong pointers to the inner value.
+	///
+	/// See [`Weak::strong_count()`] for more information.
+	/// ```
+	#[inline]
+	pub fn strong_count(&self) -> usize {
+		self.0.strong_count()
+	}
+
+	/// Gets the number of [`Weak`] pointers to this allocation.
+	///
+	/// See [`Weak::strong_count()`] for more information.
+	#[inline]
+	pub fn weak_count(&self) -> usize {
+		self.0.weak_count()
+	}
+}
+
+impl<T: ?Sized> fmt::Debug for PinWeak<T> {
+	#[inline]
+	fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+		write!(f, "Pin<(Weak)>")
+	}
+}
+
+impl<T: ?Sized> fmt::Pointer for PinWeak<T> {
+	/// Formats the value using the given formatter.
+	///
+	/// If the `#` flag is used, the state (weak) is written after the address.
+	fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+		if f.alternate() {
+			fmt::Pointer::fmt(&self.0.as_ptr(), f)?;
+			f.write_str(" [weak]")
+		} else {
+			fmt::Pointer::fmt(&self.0.as_ptr(), f)
+		}
+	}
+}
+
+impl<T: ?Sized> Clone for PinWeak<T> {
+	/// Creates another pinned weak reference for the same value.
+	///
+	/// See [`Weak::clone()`] for more information.
+	#[inline]
+	fn clone(&self) -> Self {
+		Self(self.0.clone())
+	}
+}
