@@ -60,8 +60,7 @@ impl PartialEq for ThreadId {
 	#[inline]
 	fn eq(&self, other: &Self) -> bool {
 		match (self.0, other.0) {
-			(SENITEL, _) => false,
-			(_, SENITEL) => false,
+			(SENITEL, _) | (_, SENITEL) => false,
 			(a, b) => a == b,
 		}
 	}
